import { world, system } from "@minecraft/server";

async function breakAdjacentBlocks(x, y, z, dimension, blockType) {
    const stack = [{ x, y, z }];
    const directions = [
        { x: 1, y: 0, z: 0 },
        { x: -1, y: 0, z: 0 },
        { x: 0, y: 1, z: 0 },
        { x: 0, y: -1, z: 0 },
        { x: 0, y: 0, z: 1 },
        { x: 0, y: 0, z: -1 },
        // 斜め方向
        { x: 1, y: 1, z: 0 },
        { x: 1, y: -1, z: 0 },
        { x: -1, y: 1, z: 0 },
        { x: -1, y: -1, z: 0 },
        { x: 1, y: 0, z: 1 },
        { x: 1, y: 0, z: -1 },
        { x: -1, y: 0, z: 1 },
        { x: -1, y: 0, z: -1 },
        { x: 0, y: 1, z: 1 },
        { x: 0, y: 1, z: -1 },
        { x: 0, y: -1, z: 1 },
        { x: 0, y: -1, z: -1 },
        { x: 1, y: 1, z: 1 },
        { x: 1, y: 1, z: -1 },
        { x: 1, y: -1, z: 1 },
        { x: 1, y: -1, z: -1 },
        { x: -1, y: 1, z: 1 },
        { x: -1, y: 1, z: -1 },
        { x: -1, y: -1, z: 1 },
        { x: -1, y: -1, z: -1 },
    ];

    while (stack.length > 0) {
        const currentBlock = stack.pop();
        const blockInstance = dimension.getBlock(currentBlock);
        if (blockInstance.typeId === blockType) {
            await dimension.runCommandAsync(`setblock ${currentBlock.x} ${currentBlock.y} ${currentBlock.z} air [] destroy`);

            for (const direction of directions) {
                const adjacentBlock = {
                    x: currentBlock.x + direction.x,
                    y: currentBlock.y + direction.y,
                    z: currentBlock.z + direction.z
                };

                if (dimension.getBlock(adjacentBlock).typeId === blockType) {
                    stack.push(adjacentBlock);
                }
            }
        }
    }
}

world.beforeEvents.playerBreakBlock.subscribe(event => {
    const { player, block } = event;
    if (!player.isSneaking) return;

    const blockType = block.typeId;
    const validSuffixes = [
        "ancient_debris",
        "stem",
        "log",
        "ore"
    ];

    if (validSuffixes.some(suffix => blockType.endsWith(suffix))) {
        const dimension = player.dimension;
        breakAdjacentBlocks(block.location.x, block.location.y, block.location.z, dimension, blockType);
    }
});